/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

// derived from Houndini sample: SOP_Surface

/** @file */

#ifndef __houdini_HoudiniSOP_h__
#define __houdini_HoudiniSOP_h__

namespace fe
{

/**************************************************************************//**
	@brief Houdini SOP node

	@ingroup houdini
*//***************************************************************************/
class HoudiniSOP:
	public SOP_Node,
	public MetaPlugin
{
	public:

	class OpOperator:
		public OP_Operator
	{
		public:
								OpOperator(const char *name,
										const char *english,
										OP_Constructor construct,
										PRM_Template *templates,
										unsigned min_sources,
										unsigned max_sources,
										CH_LocalVariable *variables,
										unsigned flags,
										const char **inputlabels,
										sp<Catalog> a_spCatalog);

								//* as OP_Operator
	virtual	bool				getOpHelpURL(UT_String& a_rUtString);

		private:
			String				m_helpUrl;
	};

								HoudiniSOP(OP_Network *pNetwork,
										const char *name,OP_Operator *pOp);
virtual							~HoudiniSOP(void);

static	OP_Node*				create(OP_Network *pNetwork,const char *name,
										OP_Operator *pOp);

static	int						discoverOperators(OP_OperatorTable *pTable);

								///	@internal
static	int						reloadCallback(void *pData,int index,float time,
										const PRM_Template* pTemplate);

								///	@internal
		void					brush(GU_Detail* pGdp,WindowEvent& a_rEvent,
										SpatialVector& a_rRayOrigin,
										SpatialVector& a_rRayDirection);

		void					relayErrors(void);
const	String					prompt(void);
const	String					idleEvents(void);
const	String					brushVisible(void);

	protected:
								//* as SOP_Node
virtual OP_ERROR				cookMySop(OP_Context& a_rContext);
virtual	OP_ERROR				cookMyGuide1(OP_Context& a_rContext);
virtual	OP_ERROR				save(ostream& a_rStreamOut,
										const OP_SaveFlags& a_rFlags,
										const char* a_pathPrefix);
virtual	bool					load(UT_IStream& a_rStreamIn,
										const char* a_extension,
										const char* a_path);

								//* as OP_Parameters
virtual	unsigned int			disableParms(void);


	private:
		void					createOperator(String a_operatorName,
										Real a_time);
		void					updateCatalog(Real time,BWORD a_doInput);
		void					resignalBrush(void);

		HoudiniContext					m_houdiniContext;
		String							m_brushName;
		String							m_loadedState;
		double							m_frame;

		OP_Operator*					m_pOpOperator;
		GU_Detail*						m_pGdp;
};

} // namespace

#endif /* __houdini_HoudiniSOP_h__ */
