/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __surface_SurfaceAccessorBase_h__
#define __surface_SurfaceAccessorBase_h__

namespace fe
{

/**************************************************************************//**
    @brief Common Functionality for Accessor Surface

	@ingroup surface
*//***************************************************************************/
class FE_DL_EXPORT SurfaceAccessorBase:
	virtual public SurfaceAccessorI
{
	public:
						SurfaceAccessorBase(void)
						{	setName("SurfaceAccessorBase"); }
virtual					~SurfaceAccessorBase(void)							{}

						//* as SurfaceAccessorI
virtual	String			type(void) const						{ return ""; }
virtual	U32				count(void) const
						{
							return m_spRecordArray.isValid()?
									m_spRecordArray->length(): 0;
						}
virtual	U32				subCount(U32 index) const				{ return 0; }

virtual	void			set(U32 a_index,U32 a_subIndex,String a_string)		{}
virtual	void			set(U32 a_index,String a_string)
						{	set(a_index,U32(0),a_string); }
virtual	String			string(U32 index,U32 subIndex=0)		{ return ""; }

virtual	void			set(U32 a_index,U32 a_subIndex,I32 a_integer)		{}
virtual	void			set(U32 a_index,I32 a_integer)
						{	set(a_index,U32(0),a_integer); }
virtual	I32				integer(U32 index,U32 subIndex=0)		{ return 0; }

virtual	I32				duplicate(U32 index,U32 subIndex=0)		{ return -1; }

virtual	void			append(U32 a_index,I32 a_integer)					{}
virtual	void			append(I32 a_integer)
						{	append(U32(0),a_integer); }

virtual	void			remove(U32 a_index,I32 a_integer)					{}
virtual	void			remove(I32 a_integer)
						{	remove(U32(0),a_integer); }

virtual	void			set(U32 a_index,U32 a_subIndex,Real a_real)			{}
virtual	void			set(U32 a_index,Real a_real)
						{	set(a_index,U32(0),a_real); }
virtual	Real			real(U32 index,U32 subIndex=0)		{ return Real(0); }

virtual	void			set(U32 a_index,U32 a_subIndex,
								const SpatialVector& a_vector)
						{
							m_surfacePointRV.setIndex(a_index);

							switch(m_attribute)
							{
								case SurfaceAccessibleI::e_generic:
									break;
								case SurfaceAccessibleI::e_position:
									m_surfacePointRV.location()=a_vector;
									return;
								case SurfaceAccessibleI::e_normal:
									m_surfacePointRV.up()=a_vector;
									return;
								default:
									return;
							}
							if(m_attrName=="uv")
							{
								m_surfacePointRV.texCoord()=a_vector;
							}
						}
virtual	void			set(U32 a_index,const SpatialVector& a_vector)
						{	set(a_index,U32(0),a_vector); }
virtual	SpatialVector	spatialVector(U32 a_index,U32 subIndex=0)
						{
							m_surfacePointRV.setIndex(a_index);

							switch(m_attribute)
							{
								case SurfaceAccessibleI::e_generic:
									break;
								case SurfaceAccessibleI::e_position:
									return m_surfacePointRV.location();
								case SurfaceAccessibleI::e_normal:
									return m_surfacePointRV.up();
								default:
									return SpatialVector(Real(0),
											Real(0),Real(0));
							}

							if(m_attrName=="uv")
							{
								return m_surfacePointRV.texCoord();
							}

							return SpatialVector(Real(0),Real(0),Real(0));
						}

virtual	void			fragmentWith(SurfaceAccessibleI::Element a_element,
								String a_attributeName);
virtual	U32				fragmentCount(void) const
						{	return m_fragmentMap.size(); }
virtual	String			fragment(U32 a_index) const
						{
							AutoHashMap<String,Fragment>::const_iterator it=
									m_fragmentMap.begin();
							for(U32 m=0;m<a_index;m++)
							{
								if(it==m_fragmentMap.end())
								{
									return String();
								}
								it++;
							}
							if(it==m_fragmentMap.end())
							{
								return String();
							}
							return it->first;
						}
virtual	BWORD			filterWith(String a_filterString,sp<FilterI>& rspFilter) const
						{
							AutoHashMap<String,Fragment>::const_iterator it=
									m_fragmentMap.find(a_filterString);
							if(it==m_fragmentMap.end())
							{
								rspFilter=NULL;
								return FALSE;
							}
							sp<FragmentFilter> spFragmentFilter=rspFilter;
							if(spFragmentFilter.isNull())
							{
								spFragmentFilter=
									sp<FragmentFilter>(new FragmentFilter());
								FEASSERT(spFragmentFilter.isValid());
							}
							spFragmentFilter->setFragment(&(it->second));
							rspFilter=spFragmentFilter;
							return TRUE;
						}

		void			setSurfaceAccessible(
								sp<SurfaceAccessibleI> a_spSurfaceAccessibleI)
						{	m_spSurfaceAccessibleI=a_spSurfaceAccessibleI; }

virtual	SurfaceAccessibleI::Element		element(void)	{ return m_element; }
virtual	SurfaceAccessibleI::Attribute	attribute(void)	{ return m_attribute; }

virtual	String			attributeName(void)		{ return m_attrName; }

		BWORD			bind(SurfaceAccessibleI::Element a_element,
								SurfaceAccessibleI::Attribute a_attribute)
						{
							m_attribute=a_attribute;

							String name;
							switch(a_attribute)
							{
								case SurfaceAccessibleI::e_generic:
								case SurfaceAccessibleI::e_position:
									name="P";
									break;
								case SurfaceAccessibleI::e_normal:
									name="N";
									break;
								case SurfaceAccessibleI::e_vertices:
									m_attrName="vertices";
									FEASSERT(a_element==
											SurfaceAccessibleI::e_primitive);
									m_element=a_element;
									return TRUE;
								case SurfaceAccessibleI::e_properties:
									m_attrName="properties";
									FEASSERT(a_element==
											SurfaceAccessibleI::e_primitive);
									m_element=a_element;
									return TRUE;
							}
							return bindInternal(a_element,name);
						}
		BWORD			bind(SurfaceAccessibleI::Element a_element,
								const String& a_name)
						{
							m_attribute=SurfaceAccessibleI::e_generic;
							return bindInternal(a_element,a_name);
						}

		void			bind(WeakRecord& a_rRecord)
						{	m_record=a_rRecord; }

	protected:

		SurfaceAccessibleI::Element		m_element;
		SurfaceAccessibleI::Attribute	m_attribute;
		String							m_attrName;

	private:

		BWORD			bindInternal(SurfaceAccessibleI::Element a_element,
								const String& a_name);

		WeakRecord				m_record;
		sp<RecordArray>			m_spRecordArray;
		SurfacePoint			m_surfacePointRV;

		sp<SurfaceAccessibleI>	m_spSurfaceAccessibleI;

	class Fragment
	{
		public:
			std::vector<U32>	m_indexArray;
	};
	class FragmentFilter: public SurfaceAccessorI::FilterI
	{
		public:
					FragmentFilter(void)									{}
	virtual			~FragmentFilter(void)									{}

	virtual	I32		filterCount(void) const
					{
						return m_pFragment?
								m_pFragment->m_indexArray.size(): -1;
					}
	virtual	I32		filter(I32 a_filterIndex) const
					{
						if(!m_pFragment)
						{
							return -1;
						}
						const std::vector<U32>& rIndexArray=
								m_pFragment->m_indexArray;
						if(a_filterIndex>=I32(rIndexArray.size()))
						{
							return -1;
						}
						return rIndexArray[a_filterIndex];
					}

					/// @internal
			void	setFragment(const Fragment* a_pFragment)
					{	m_pFragment=a_pFragment; }

		private:
			const Fragment*			m_pFragment;
	};

	AutoHashMap<String,Fragment>	m_fragmentMap;
};

} // namespace

#endif /* __surface_SurfaceAccessorBase_h__ */
